@@ -15,11 +15,23 @@ namedList <- function (...) {
     setNames(L, nm)
 }
 
-##' @importFrom stats reformulate
 RHSForm <- function(form,as.form=FALSE) {
-    rhsf <- form[[length(form)]]
-    if (as.form) as.formula(substitute(~F,list(F=rhsf)),
-                            env=environment(form)) else rhsf
+    if (!as.form) return(form[[length(form)]])
+    if (length(form)==2) return(form)  ## already RHS-only
+    ## by operating on RHS in situ rather than making a new formula
+    ## object, we avoid messing up existing attributes/environments etc.
+    form[[2]] <- NULL
+    ## assumes response is *first* variable (I think this is safe ...)
+    if (length(vars <- attr(form,"variables"))>0) {
+        attr(form,"variables") <- vars[-2]
+    }
+    if (is.null(attr(form,"response"))) {
+        attr(form,"response") <- 0
+    }
+    if (length(facs <- attr(form,"factors"))>0) {
+        attr(form,"factors") <- facs[-1,]
+    }
+    return(form)
 }
 
 `RHSForm<-` <- function(formula,value) {
@@ -34,12 +46,16 @@ RHSForm <- function(form,as.form=FALSE) {
 ##                response=response)
 ## }
 
+sumTerms <- function(termList) {
+    Reduce(function(x,y) makeOp(x,y,op=quote(`+`)),termList)
+}
+
 ## better version -- operates on language objects (no deparse())
 reOnly <- function(f,response=FALSE,bracket=TRUE) {
     ff <- f
     if (bracket)
         ff <- lapply(findbars(ff),makeOp,quote(`(`)) ## bracket-protect terms
-    ff <- Reduce(function(x,y) makeOp(x,y,op=quote(`+`)),ff)
+    ff <- sumTerms(ff)
     if (response && length(f)==3) {
         form <- makeOp(f[[2]],ff,quote(`~`))
     } else {
@@ -52,12 +68,16 @@ reOnly <- function(f,response=FALSE,bracket=TRUE) {
 ## FIXME: would be nice to have multiple dispatch, so
 ## (arg,op) gave unary, (arg,arg,op) gave binary operator
 makeOp <- function(x,y,op=NULL) {
-    if (is.null(op)) {  ## unary
-        substitute(OP(X),list(X=x,OP=y))
+    if (is.null(op) || missing(y)) {  ## unary
+        if (is.null(op)) {
+            substitute(OP(X),list(X=x,OP=y))
+        } else {
+            substitute(OP(X),list(X=x,OP=op))
+        }
     } else substitute(OP(X,Y), list(X=x,OP=op,Y=y))
 }
 
-## combines the right-hand sides of two formulas
+## combines the right-hand sides of two formulas, or a formula and a symbol
 ## @param f1 formula #1
 ## @param f2 formula #2
 ## @examples
@@ -66,10 +86,14 @@ makeOp <- function(x,y,op=NULL) {
 ## addForm0(~x,~y)
 ## }
 ## @keywords internal
-addForm0 <- function(f1,f2) {
-  if (length(f2)==3) warning("discarding LHS of second argument")
-  RHSForm(f1) <- makeOp(RHSForm(f1),RHSForm(f2),quote(`+`))
-  return(f1)
+addForm0 <- function(f1,f2,naked=FALSE) {
+    tilde <- as.symbol("~")
+    if (!identical(head(f2),tilde)) {
+        f2 <- makeOp(f2,tilde)
+    }
+    if (length(f2)==3) warning("discarding LHS of second argument")
+    RHSForm(f1) <- makeOp(RHSForm(f1),RHSForm(f2),quote(`+`))
+    return(f1)
 }
 
 ##' Combine right-hand sides of an arbitrary number of formulas
@@ -141,11 +165,14 @@ expandAllGrpVar <- function(bb) {
     }
 }
 
-## sugar -- ???? this returns '~'  ???
+## sugar: this returns the operator, whether ~ or something else
 head.formula <- head.call <- function(x, ...) {
     x[[1]]
 }
 
+## sugar: we can call head on a symbol and get back the symbol
+head.name <- function(x) { x }
+
 ##' (f)ind (b)ars e(x)tended: recursive
 ##'
 ##' @param term a formula or piece of a formula
@@ -369,17 +396,74 @@ anySpecial <- function(term) {
     any(findReTrmClasses() %in% all.names(term))
 }
 
-## does the formula contain a particular value?
-## inForm(z~.,quote(.))
-## inForm(z~y,quote(.))
-## inForm(z~a+b+c,quote(c))
-## inForm(z~a+b+(d+e),quote(c))
+##' test formula: does it contain a particular element?
+##' @rdname formFuns
+##' @examples
+##' inForm(z~.,quote(.))
+##' inForm(z~y,quote(.))
+##' inForm(z~a+b+c,quote(c))
+##' inForm(z~a+b+(d+e),quote(c))
+##' f <- ~ a + offset(x)
+##' f2 <- z ~ a
+##' inForm(f,quote(offset))
+##' inForm(f2,quote(offset))
+##' @export
+##' @keywords internal
 inForm <- function(form,value) {
     if (any(sapply(form,identical,value))) return(TRUE)
     if (all(sapply(form,length)==1)) return(FALSE)
-    return(any(sapply(form,inForm,value)))
+    return(any(vapply(form,inForm,value,FUN.VALUE=logical(1))))
 }
 
+##' extract terms with a given head from an expression/formula
+##' @rdname formFuns
+##' @param term expression/formula
+##' @param value head of terms to extract
+##' @return a list of expressions
+##' @examples
+##' extractForm(~a+offset(b),quote(offset))
+##' extractForm(~c,quote(offset))
+##' extractForm(~a+offset(b)+offset(c),quote(offset))
+##' @export
+##' @keywords internal
+extractForm <- function(term,value) {
+    if (!inForm(term,value)) return(NULL)
+    if (is.name(term) || !is.language(term)) return(NULL)
+    if (identical(head(term),value)) {
+        return(term)
+    }
+    if (length(term) == 2) {
+        return(extractForm(term[[2]],value))
+    }
+    return(c(extractForm(term[[2]],value),
+             extractForm(term[[3]],value)))
+}
+
+##' return a formula/expression with a given value stripped, where
+##' it occurs as the head of a term
+##' @rdname formFuns
+##' @examples 
+##' dropHead(~a+offset(b),quote(offset))
+##' dropHead(~a+poly(x+z,3)+offset(b),quote(offset))
+##' @export
+##' @keywords internal
+dropHead <- function(term,value) {
+    if (!inForm(term,value)) return(term)
+    if (is.name(term) || !is.language(term)) return(term)
+    if (identical(head(term),value)) {
+        return(term[[2]])
+    }
+    if (length(term) == 2) {
+        return(dropHead(term[[2]],value))
+    } else  if (length(term) == 3) {
+        term[[2]] <- dropHead(term[[2]],value)
+        term[[3]] <- dropHead(term[[3]],value)
+        return(term)
+    } else stop("length(term)>3")
+}
+
+
+## UNUSED (same function as drop.special2?)
 # drop.special(x~a + b+ offset(z))
 drop.special <- function(term,value=quote(offset)) {
     if (length(term)==2 && identical(term[[1]],value)) return(NULL)
@@ -401,16 +485,28 @@ drop.special <- function(term,value=quote(offset)) {
     }
 }
 
+##' drop terms matching a particular value from an expression
+##' @rdname formFuns
 ## from Gabor Grothendieck: recursive solution
 ## http://stackoverflow.com/questions/40308944/removing-offset-terms-from-a-formula
+##' @param x formula
+##' @param value term to remove from formula
+##' @param preserve (integer) retain the specified occurrence of "value"
+##' @keywords internal
 drop.special2 <- function(x, value=quote(offset), preserve = NULL) {
   k <- 0
   proc <- function(x) {
     if (length(x) == 1) return(x)
-    if (x[[1]] == value && !((k<<-k+1) %in% preserve)) return(x[[1]])
+    if (x[[1]] == value && !((k <<- k+1) %in% preserve)) return(x[[1]])
     replace(x, -1, lapply(x[-1], proc))
   }
-  update(proc(x), substitute(. ~ . - x,list(x=value)))
+  ## handle 1- and 2-sided formulas
+  if (length(x)==2) {
+      newform <- substitute(~ . -x, list(x=value))
+  } else {
+      newform <- substitute(. ~ . - x, list(x=value))
+  }
+  return(update(proc(x), newform))
 }
 
 ## Sparse Schur complement (Marginal of precision matrix)
@@ -427,3 +523,95 @@ GMRFmarginal <- function(Q, i, ...) {
         solve(Q0, as.matrix(Q[i0, i1, drop = FALSE]))
     ans
 }
+
+# n.b. won't work for terms with more than 2 args ...
+# @examples
+# replaceForm(quote(a(b+x*c(y,z))),quote(y),quote(R))
+# ss <- ~(1 | cask:batch) + (1 | batch)
+# replaceForm(ss,quote(cask:batch),quote(batch:cask))
+replaceForm <- function(term,target,repl) {
+    if (identical(term,target)) return(repl)
+    if (!inForm(term,target)) return(term)
+    if (length(term) == 2) {
+        return(substitute(OP(x),list(OP=term[[1]],x=replaceForm(term[[2]],target,repl))))
+    }
+    return(substitute(OP(x,y),list(OP=term[[1]],
+                                   x=replaceForm(term[[2]],target,repl),
+                                   y=replaceForm(term[[3]],target,repl))))
+}
+
+parallel_default <- function(parallel=c("no","multicore","snow"),ncpus=1) {
+    ##  boilerplate parallel-handling stuff, copied from lme4
+    if (missing(parallel)) parallel <- getOption("profile.parallel", "no")
+    parallel <- match.arg(parallel)
+    do_parallel <- (parallel != "no" && ncpus > 1L)
+    if (do_parallel && parallel == "multicore" &&
+        .Platform$OS.type == "windows") {
+        warning("no multicore on Windows, falling back to non-parallel")
+        parallel <- "no"
+    }
+    return(list(parallel=parallel,do_parallel=do_parallel))
+}
+
+##' translate vector of correlation parameters to correlation values,
+##' following the definition at \url{http://kaskr.github.io/adcomp/classUNSTRUCTURED__CORR__t.html}:
+##' if \eqn{L} is the lower-triangular matrix with 1 on the diagonal and the correlation parameters in the lower triangle, then the correlation matrix is defined as \eqn{\Sigma = D^{-1/2} L L^\top D^{-1/2}}{Sigma = sqrt(D) L L' sqrt(D)}, where \eqn{D = \textrm{diag}(L L^\top)}{D = diag(L L')}. For a single correlation parameter \eqn{\theta_0}{theta0}, this works out to \eqn{\rho = \theta_0/\sqrt{1+\theta_0^2}}{rho = theta0/sqrt(1+theta0^2)}.
+##' @param theta vector of internal correlation parameters
+##' @return a vector of correlation values
+##' @examples
+##' th0 <- 0.5
+##' stopifnot(all.equal(get_cor(th0),th0/sqrt(1+th0^2)))
+##' get_cor(c(0.5,0.2,0.5))
+##' @export
+get_cor <- function(theta) {
+    n <- round((1  + sqrt(1+8*length(theta)))/2) ## dim of cor matrix
+    L <- diag(n)
+    L[lower.tri(L)] <- theta
+    cL <- tcrossprod(L)
+    Dh <- diag(1/sqrt(diag(cL)))
+    cc <- Dh %*% cL %*% Dh
+    return(cc[lower.tri(cc)])
+}
+
+match_which <- function(x,y) {
+    which(sapply(y,function(z) x %in% z))
+}
+
+## reassign predvars to have term vars in the right order,
+##  but with 'predvars' values inserted where appropriate
+fix_predvars <- function(pv,tt) {
+    if (length(tt)==3) {
+        ## convert two-sided to one-sided formula
+        tt <- RHSForm(tt, as.form=TRUE)
+    }
+    ## ugh, deparsing again ...
+    tt_vars <- vapply(attr(tt,"variables"),deparse,character(1))[-1]
+    ## remove terminal paren - e.g. match term poly(x, 2) to
+    ##   predvar poly(x, 2, <stuff>)
+    ## beginning of string, including open-paren, colon
+    ##  and *first* comma but not arg ... 
+    init_regexp <- "^([(:_.[:alnum:]]+).*"
+    tt_vars_short <- gsub(init_regexp,"\\1",tt_vars)
+    if (is.null(pv) || length(tt_vars)==0) return(NULL)
+    new_pv <- quote(list())
+    ## maybe multiple variables per pv term ... [[-1]] ignores head
+    ## FIXME: test for really long predvar strings ????
+    pv_strings <- vapply(pv,deparse,FUN.VALUE=character(1),
+                         width.cutoff=500)[-1]
+    pv_strings <- gsub(init_regexp,"\\1",pv_strings)
+    for (i in seq_along(tt_vars)) {
+        w <- match(tt_vars_short[[i]],pv_strings)
+        if (!is.na(w)) {
+            new_pv[[i+1]] <- pv[[w+1]]
+        } else {
+            ## insert symbol from term vars
+            new_pv[[i+1]] <- as.symbol(tt_vars[[i]])
+        }
+    }
+    return(new_pv)
+}
+
+has.random <- function(x) {
+    pl <- getParList(x)
+    return(length(unlist(pl[grep("^theta",names(pl))]))>0)
+}
